/* 
 * Copyright (C) 2002 by Greg Gardner
 * 
 * This file is part of elseed, a caller-id program for your TiVo.
 * 
 * elseed is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * elseed is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: elseed.c,v 1.4 2002/04/23 23:53:28 greg Exp $
 */


#include "elseed.h"
#include "osd.h"

/* loop til we die */
int STOP = FALSE; 

/* incoming caller id flag */
int INC_CID = FALSE;
int saw_name = FALSE;
int saw_nmbr = FALSE;

/* the caller id structure */
caller_id cid;

int osd_x=X_POS_DEFAULT, osd_y=Y_POS_DEFAULT;
int fg_color=FG_COLOR_DEFAULT, bg_color=BG_COLOR_DEFAULT;
int debug = 0;
char modem_device[255];
char modem_init[255];
char log_file[255];

/* the main shiznit */
int main (int argc, char *argv[])  {
    int fd, result, c;
    char inbuff[SZ_BUFF], outbuff[SZ_BUFF];
    char *sloc, *eloc;
    struct termios oldtio, newtio;
    struct stat statbuf;

    (void) signal (SIGINT, getout);	/* exit cleanly on ctrl-c */ 

    SetupTextOSD();

    strncpy(modem_device, MODEMDEVICE_DEFAULT, 255);
    strncpy(modem_init, MODEMINIT_DEFAULT, 253);
    strcat(modem_init, "\r\n");
    strncpy(log_file, LOG_FILE_DEFAULT, 255);

    while ((c = getopt(argc, argv, "x:y:b:f:i:d:l:Dh")) != -1) {
        switch (c) {
            case 'x':
                osd_x = atoi(optarg);
                break;
            case 'y':
                osd_y = atoi(optarg);
                break;
            case 'f':
                fg_color = atoi(optarg);
                break;
            case 'b':
                fg_color = atoi(optarg);
                break;
            case 'd':
                strncpy(modem_device, optarg, 255);
                break;
            case 'i':
                strncpy(modem_init, optarg, 253);
                strcat(modem_init, "\r\n");
                break;
            case 'l':
                strncpy(log_file, optarg, 255);
                break;
            case 'D':
                debug = 1;
                break;
            case 'h':
                usage();
            default:
                usage();
        };
    }

start:

    fd = open (modem_device, O_RDWR | O_NOCTTY);
    if (fd < 0)  {
        perror (modem_device); 
        exit (-1);
    }
    
    save_modem_settings(fd, &oldtio, &newtio);

    printf ("Initializing modem...\n");
    write (fd, modem_init, strlen (modem_init));

    memset (&outbuff, 0, SZ_BUFF);			/* init the buff */
    while (STOP == FALSE)  {			/* loop for input */
        if (wait_for_input(fd, 60 * 1000)) {
            //sleep(1); /* For safe keeping so we don't get stuck in a loop */
            if (stat("/var/tmp/modemlock", &statbuf) == 0) {
                /* Tivo has a lock on the modem, don't 
                   do a read, and reopen the port from 
                   scratch after tivo has had its way with it */
                printf("Tivo's making a call, won't step on his toes\n");
                while (stat("/var/tmp/modemlock", &statbuf) == 0) {
                    if (STOP == TRUE) {
                        break;
                    }
                    sleep(10);
                }
                close(fd);
                printf("Restarting...\n");
                goto start;
            }
            while ((result = read(fd, inbuff, SZ_BUFF - strlen(outbuff) - 1)) > 0) {    /* grab the buff */
                inbuff[result] = 0;			/* mark end of buff */
                strcat (outbuff, inbuff);		/* build our str */
            }
            sloc = outbuff;
            while (sloc != NULL)  {	/* at end of str */
                if ((eloc = strstr (sloc, "\r\n")) != NULL) {
                    if (parse (sloc, eloc - sloc))  {   /* parse it out */
                        printit ();             /* dump output */
                        logit (log_file);       /* log output */
                        osdit();
                        close(fd);
                        printf("Restarting...\n");
                        goto start;
                    }
                    sloc = eloc + 2;
                }
                else {
                    break;
                }
            }
            memset (outbuff, 0, SZ_BUFF);	/* reset the buff */
        }
    }

    tcsetattr(fd, TCSANOW, &oldtio);  /* set the port back */
    FreeTextOSD();
    printf ("Exited cleanly.\n");
    return (0);
}

void save_modem_settings(int fd, struct termios *oldtio, struct termios *newtio) {
    tcgetattr (fd, oldtio);  /* save current port settings */
	
    memset (newtio, 0, sizeof(*newtio));
    newtio->c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
    newtio->c_iflag = IGNPAR;
    newtio->c_oflag = 0;

    /* set input mode (non-canonical, no echo,...) */
    newtio->c_lflag = 0;

    newtio->c_cc[VTIME] = 50;	/* read after x tenths of a second */
    newtio->c_cc[VMIN] = 0;     /* read after x characters */

    tcflush (fd, TCIFLUSH);
    tcsetattr (fd, TCSANOW, newtio);
    return;
}

int wait_for_input(fd, msecs) {
    fd_set rfds;
    struct timeval timeout, *tptr;
    int ret;

    FD_ZERO( &rfds );
    FD_SET( fd, &rfds );
    if ( msecs >= 0 )
    {
        timeout.tv_sec = msecs / 1000;
        timeout.tv_usec = (msecs % 1000) * 1000;
        tptr = &timeout;
    }
    else
        tptr = NULL;

    ret = select( FD_SETSIZE, &rfds, NULL, NULL, tptr );
    return ret > 0;
}

/* parse it out to get the good stuff */
int parse (char *input, int len)  {
    int	ret_val = 0;
    char *dup;

    dup = malloc((len + 1)*sizeof(char));
    strncpy (dup, input, len);
    dup[len] = 0;

    if (debug) {
        printf("DEBUG: %s\n", dup);
    }

    /* printf ("Debug: parse: input = %s\n", input); */

    if (strncmp (dup, "RING", 4) == 0)  {
        printf ("Modem: RING\n");
    }

    if (strncmp (dup, "OK", 2) == 0)  {
        printf ("Modem: OK\n");
    }

    if (strncmp (dup, "ERROR", 5) == 0)  {
        printf ("Modem: ERROR\n");
    }

    /* w00t! caller id info coming in, flip the flag on */
    if (strstr (dup, CID_DATE) != NULL)  {
        memset (&cid, 0, sizeof (caller_id)); /* init struct */
        INC_CID = TRUE;
    }

    /* copy all strings from the end of their flag to null */
    if (INC_CID == TRUE)  {
        if (strstr (dup, CID_DATE) != NULL)  {
            strcpy (cid.date, dup + strlen (CID_DATE));
        }
        if (strstr (dup, CID_TIME) != NULL)  {
            strcpy (cid.time, dup + strlen (CID_TIME));
        }
        if (strstr (dup, CID_NAME) != NULL)  {
            saw_nmbr = TRUE;
            strcpy (cid.name, dup + strlen (CID_NAME));
        }
        if (strstr (dup, CID_NMBR) != NULL)  {
            saw_name = TRUE;
            strcpy (cid.nmbr, dup + strlen (CID_NMBR));
        }
        if (saw_name && saw_nmbr) {
            INC_CID = FALSE;
            saw_name = FALSE;
            saw_nmbr = FALSE;
            ret_val = TRUE;
        }
    }

    free (dup);
    return (ret_val);
}

void osdit() {
    int x, rx, y, ry, max_length;
    max_length = strlen(cid.name) > strlen(cid.nmbr) ? strlen(cid.name) : strlen(cid.nmbr);
    if (osd_x >= 0) {
        rx = osd_x;
    }
    else {
        rx = 35 + osd_x - max_length;
    }
    if (osd_y >= 0) {
        ry = osd_y;
    }
    else {
        ry = 15 + osd_y;
    }
    x = rx; y = ry;
    ClearOSD();
    DrawString(&x, &y, cid.name, fg_color, bg_color);
    x = rx; y = ry + 1;
    DrawString(&x, &y, cid.nmbr, fg_color, bg_color);
    DrawOSD();
    sleep(10);
    ClearOSD();
    DrawOSD();
}

void printit ()  {
    printf ("--Call Logged--\n");
    printf ("  Date:   %s\n", cid.date);
    printf ("  Time:   %s\n", cid.time);
    printf ("  Name:   %s\n", cid.name);
    printf ("  Number: %s\n\n", cid.nmbr);
}

void logit (char *logfile)  {
    FILE	*file;
    time_t	unix_time;
    
    unix_time = time (NULL);

    if ((file = fopen (logfile, "a")) == NULL)  {
        printf ("Failed to write to %s.\n", logfile);
    }  else  {
        fprintf (file, "%s\t%s\t%s\t%s\t%ld\n",
                 cid.date,
                 cid.time,
                 cid.name,
                 cid.nmbr,
                 unix_time);
    }
    fclose (file);
}

void usage() {
    printf("Usage: elseed [options]\n");
    printf("Options:\n");
    printf("    -x N    set x coordinate to N [default %d]\n", X_POS_DEFAULT);
    printf("    -y N    set y coordinate to N [default %d]\n", Y_POS_DEFAULT);
    printf("    -f N    set foreground color to N [default %d]\n", FG_COLOR_DEFAULT);
    printf("    -b N    set background color to N [default %d]\n", BG_COLOR_DEFAULT);
    printf("    -d S    set modem device to S [default '%s']\n", MODEMDEVICE_DEFAULT);
    printf("    -i S    set modem init string to S [default '%s']\n", MODEMINIT_DEFAULT);
    printf("    -l S    set log file to S [default '%s']\n", LOG_FILE_DEFAULT);
    printf("    -D      put elseed in debug mode\n");
    printf("    -h      show this help\n");
    exit(0);
}

/* let ya know we're exiting cleanly */
void getout (int sig)  {
    printf ("Caught signal %d, standby while exiting...\n", sig);
    STOP = TRUE;
}
